# -*- coding: utf-8 -*- 
from os import path as osp
from os import unlink

import autopath
from filtrage.guesser import *
from filtrage.config import KEYWORDS

DATA = osp.join(autopath.HERE, 'data')
TRAIN_DB = osp.join(DATA, 'training')
BAYES_DB = osp.join(DATA,'bayesdb', 'guesser.bay')

def init_database(usemeta=False):
    """creation des bases bayesiennes (fichiers en .bay)
    définies par les mots-clef
    """
    if usemeta:
        train_guesser(keywords=KEYWORDS, filename=BAYES_DB, guess_data=TRAIN_DB,
                      mode='meta')
        
    else:
        train_guesser(keywords=KEYWORDS, filename=BAYES_DB, guess_data=TRAIN_DB,
                      mode='html')
        
    assert osp.isfile(BAYES_DB)

def rm_database():
    "suppression des fichiers de bases baysiennes"
    unlink(BAYES_DB)

def test_make_guesser():
    """fabrique un classificateur et l'enregistre"""
    try:
        init_database(usemeta=True)
    finally:
        rm_database()

    try:
        init_database(usemeta=False)
    finally:
        rm_database()

def almost_equal(x, y, places=7):
    """arrondi pour comparer aproximativement deux résultats"""
    return round(y-x, places) == 0

def lst_to_dct(rates):
    """affichage des résultats sous forme de dictionnaire
    permet de passer de 
    [('porn', 0.94649927972169501), ('noporn', 0.067599721064875107)]
    [('noporn', 0.99990000000000001)]
    à un format :
    {'porn': 0.94649927972169501}
    """
    d = {}
    for name, rate in rates:
        d[name]=rate 
    return d

# def test_guesser_meta_content():
#     """teste la notation d'un classificateur
#     - apprentissage uniquement d'après le contenu des meta
#     - notation sur tout le contenu
#     """
#     init_database(usemeta=True)
#     g = Guesser()
#     g.load(BAYES_DB)
#     result = g.guess_line("fuck ass nipple sex sexe cul bite sein nichons trou du cul hard penetration")
#     result = lst_to_dct(result)
#     expected = [('porn', 0.86277472955088541), ('noporn', 0.29028106298516243)]
#     expected = lst_to_dct(expected)
#     for key, value in expected.items():
#         assert almost_equal(expected[key], result[key])
#     expected = [('porn', 0.49065666230642052), ('noporn', 0.50934333769357953)]
#     expected = lst_to_dct(expected)
#     result = g.guess_line("ceci est un exemple de texte simple sans aucune espece de contenu scabreux d'aucune sorte")
#     result = lst_to_dct(result)
#     for key, value in expected.items():
#         assert almost_equal(expected[key], result[key])

#     try:
#         rm_database()
#     except:
#         pass

# def test_guesser_meta_content_meta_rate():
#     """teste la notation d'un classificateur
#     - apprentissage uniquement d'après le contenu des meta
#     - notation uniquement sur les meta
#     """
#     init_database(usemeta=True)
#     g = Guesser()
#     g.load(BAYES_DB)
#     """exemples de résultats
#     [('porn', 0.94649927972169501), ('noporn', 0.067599721064875107)]
#     [('noporn', 0.99990000000000001)]
#     """
#     try:
#         # récupération des metas de pages
#         path = osp.join(TRAIN_DB, 'porn')
#         fname_expect = [('index.html.297', [('porn', 0.84796592811263394), ('noporn',0.59276971773419307)]),
#                      ('index.html.283', [('porn', 0.87981889599161678), ('noporn', 0.51606066021036423)]), 
#                      ('index.html.291', [('porn', 0.79124917709737475), ('noporn', 0.60887273262097585)]),
#                      ('index.html.287', [('porn', 0.87078401668496586), ('noporn', 0.64569131091504373)]),
#                      ]

#         for fname, expected in fname_expect:
#             data = unicode(file(osp.join(path, fname)).read(), 'latin-1')
#             result = g.guess_line(data)
#             result = lst_to_dct(result)
#             expected = lst_to_dct(expected)
#             for key, value in expected.items():
#                 assert almost_equal(expected[key], result[key])

#     finally:
#         rm_database()


def test_guesser_full_content():
    """teste la notation d'un classificateur"""
    init_database()
    g = Guesser()
    g.load(BAYES_DB)
    """exemples de résultats
    [('porn', 0.94649927972169501), ('noporn', 0.067599721064875107)]
    [('noporn', 0.99990000000000001)]
    """
    result = g.guess_line("fuck ass nipple sex sexe cul bite sein nichons trou du cul hard penetration", mode="txt")
    expected = [('porn', 0.98215047377065301), ('noporn', 0.045134455898390435)]
    result = lst_to_dct(result)
    expected = lst_to_dct(expected)
    for key, value in expected.items():
        assert almost_equal(expected[key], result[key])

    expected = [('noporn', 0.1179334500525932), ('porn', 0.91688132389569255)]
    result = g.guess_line("ceci est un exemple de texte simple sans aucune espece de contenu scabreux d'aucune sorte", mode="txt")
    result = lst_to_dct(result)
    expected = lst_to_dct(expected)
    for key, value in expected.items():
        assert almost_equal(expected[key], result[key])

    try:
        rm_database()
    except:
        pass
